#!/bin/bash
# IMPORTANT: use $HOME as the base for everything
notify(){
    printf "\n"
    figlet -f digital -c $1
    printf "\n"
}

modifyEtcEnvironment(){
  ADDVARS=$(cat <<- END
export LC_ALL=C
export LANG=en_US.UTF-8
\n

END
)

  (printf "${ADDVARS}" && sudo cat /etc/environment) > $HOME/temp_env && sudo mv $HOME/temp_env /etc/environment
}

WINDOW_MANAGER=$(loginctl show-session $(loginctl | grep "$USER" | awk '{print $1}') -p Desktop | awk -F"=" 'FNR == 1  {print $NF}')
echo "Desktop Session is  $WINDOW_MANAGER"

#modify /etc/environment
modifyEtcEnvironment

sudo apt-get update

#Install essential programs
sudo apt-get -y install figlet build-essential git curl
sudo apt-get -y install unclutter watchdog

notify "Installing  mpv video player,  fbi,feh, pqiv image players and scrot, grimshot screenshot tool"
sudo apt-get install -y mpv fbi feh pqiv scrot swayimg grimshot

notify "Installing networking utilities"
if nmcli general status; then
  sudo apt-get -y install ethtool nmap host
else
  sudo apt-get -y install ethtool nmap host dnsmasq hostapd
fi

notify "Get a list of WiFi devices in player-UI using iwlist cmd"
sudo apt-get -y install wireless-tools

notify "Installing node-canvas libraries"
sudo apt-get install -y build-essential libcairo2-dev libpango1.0-dev libjpeg-dev libgif-dev librsvg2-dev

notify "Installing CEC utilities"
sudo apt-get -y install cec-utils libcec-dev

if [[ ${WINDOW_MANAGER} != 'LXDE-pi-wayfire' ]]; then
  notify "Installing xterm terminal application" # not needed in new OS? - using lxterminal
  sudo apt-get install -y xterm xbindkeys
fi

if nmcli general status; then
  notify "Adding network-config symlink to /usr/bin" 
  sudo rm /usr/bin/network-config
  sudo ln -s $HOME/player2/shell-scripts/network/network-config-bookworm.sh /usr/bin/network-config
  sudo chmod +x  $HOME/player2/shell-scripts/network/network-config-bookworm.sh
  sudo chmod +x  /usr/bin/network-config
else
  notify "Adding network-config symlink to /usr/bin"
  sudo rm /usr/bin/network-config
  sudo ln -s $HOME/player2/shell-scripts/network/network-config /usr/bin/network-config
  sudo chmod +x  $HOME/player2/shell-scripts/network/network-config
  sudo chmod +x  /usr/bin/network-config
fi

notify "create directory for piSignage"
cd $HOME
mkdir -p $HOME/media
chmod -R 777 $HOME/media

mkdir -p $HOME/logs
chmod -R 777 $HOME/logs

mkdir -p $HOME/apps-data/.cache/chromium
mkdir -p $HOME/apps-data/.config/chromium
mkdir -p $HOME/apps-data/.config/chromium/Default-1
mkdir -p $HOME/apps-data/.config/chromium/weblink-0
mkdir -p $HOME/apps-data/.config/chromium/weblink-1
mkdir -p $HOME/apps-data/.config/chromium/weblink-2
mkdir -p $HOME/apps-data/.config/chromium/weblink-3
mkdir -p $HOME/apps-data/.config/chromium/weblink-4
chmod -R 777 $HOME/apps-data
mkdir $HOME/pisignage-data
echo '#!/usr/bin/env bash' > $HOME/pisignage-data/upgrade.sh

notify "SDL Ticker"
cd $HOME/player2/tools/sdl_ticker
chmod +x make.sh
./make.sh

notify "Adding evince, xdotool and wmctrl for PDF play"
if [[ ${WINDOW_MANAGER} != 'LXDE-pi-wayfire' ]]; then
  sudo apt-get -y install evince xdotool wmctrl
  sudo sed -i 's/^Exec=/# &/' /usr/share/thumbnailers/evince.thumbnailer                   #disable thumbnails
else
  sudo apt-get -y install evince
  sudo sed -i 's/^Exec=/# &/' /usr/share/thumbnailers/evince.thumbnailer                   #disable thumbnails
fi

#sudo cp $HOME/player2/shell-scripts/usb-import/pisignage-import.service /etc/systemd/system/
#sudo systemctl enable pisignage-import.service
#
#sudo cp $HOME/player2/shell-scripts/usb-import/40-pisignage-import.rules /etc/udev/rules.d/
#sudo udevadm control --reload

chmod +x $HOME/player2/shell-scripts/usb-import/import-assets.sh

# for node-canvas installation
sudo apt-get install -y libgif-dev

if grep -qi 'raspbian.*bookworm' /etc/os-release; then
  # notify "Setting up node 20.x"
  cd $HOME/Downloads
  wget https://nodejs.org/dist/v20.10.0/node-v20.10.0-linux-armv7l.tar.xz
  tar -xf node-v20.10.0-linux-armv7l.tar.xz
  cd node-v20.10.0-linux-armv7l
  sudo cp -R * /usr/local/
else
  #Installing nodejs and node modules # nodesource is deprecated
  #[nodesource](https://github.com/nodesource/distributions/blob/master/README.md#installation-instructions)
  notify "Setting up node 14.x"
  curl -sL https://deb.nodesource.com/setup_14.x | sudo bash -
  sudo apt-get install -y nodejs
fi
#sudo npm i update npm -g                    #could become incompatible with nodejs
sudo npm install pm2 -g

cd $HOME/player2
npm install

cd $HOME/player2/public/kiosk-ui
npm install


notify "welcome video"
cd $HOME
wget https://s3.amazonaws.com/pisignage/assets/pisignage/boot_video.mov
wget https://s3.amazonaws.com/pisignage/assets/pisignage/update_video.mov
wget https://s3.amazonaws.com/pisignage/assets/pisignage/update.png

# version name will of the form :    460_11_admin_2015-05-21
echo "writing version number to package.json"
PACKAGE_JSON=$HOME/player2/package.json
PI_VERSION=$(sed -n -e 's/^.*"version": //p' -e 's/[^0-9]*//g' $PACKAGE_JSON | sed 's/[^0-9]*//g')
VERSION=$(sed -n -e 's/^.*VERSION_ID=//p' -e 's/[^0-9]*//g' /etc/os-release | sed 's/[^0-9]*//g')
TODAY=`date +"%F"`
ME='script'
VERSION_NAME=${PI_VERSION}_${VERSION}_${ME}_${TODAY}

if grep -q platform_version $PACKAGE_JSON;then
    echo "line present"
    sed -i "s/.*platform_version.*/    \"platform_version\":  \"$VERSION_NAME\",/" $PACKAGE_JSON
else
    echo "line not present"
    sed -i "/.*version.*/a \    \"platform_version\":  \"$VERSION_NAME\", " $PACKAGE_JSON
fi

echo "platform_version: ${VERSION_NAME}" | sudo tee  $HOME/pisignage_version.txt

#vlc media player is not installed on debian by default
echo "installing vlc media player and pqiv for logo display"
sudo apt install vlc -y

sudo apt-get install -y pqiv

notify "installing yt-dlp for Youtube streams on MPV player"
sudo curl -L https://github.com/yt-dlp/yt-dlp/releases/latest/download/yt-dlp -o /usr/local/bin/yt-dlp
sudo chmod a+rx /usr/local/bin/yt-dlp

#notify "youtube-dl"   Not Needed in player2
#sudo -H pip install --upgrade youtube-dl # error: externally-managed-environment to be installed using sudo apt install python3-<package>
#sudo apt install youtube-dl

sudo apt-get -y update --fix-missing

#Not copied in few WM cases, so copying for all cases
notify "Auto startup script"
cp $HOME/player2/shell-scripts/start.sh $HOME/
sudo chmod 777 $HOME/start.sh

