#!/bin/bash
# IMPORTANT: use $HOME as the base for everything
notify(){
    printf "\n"
    figlet -f digital -c $1
    printf "\n"
}

sudo apt-get -y install wmctrl dconf-editor

DISPLAY_SERVER=$(loginctl show-session $(loginctl | grep "$USER" | awk '{print $1}') -p Type | awk -F"=" 'FNR == 1  {print $NF}')
echo "Windowing System is  $DISPLAY_SERVER"           # x11, wayland

if DISPLAY=:0.0 wmctrl -m ; then
    WINDOW_MANAGER=$(DISPLAY=:0.0 wmctrl -m | awk '/Name:/ {print $2}')
fi
if [ -z "$WINDOW_MANAGER" ]
then
      WINDOW_MANAGER=$(loginctl show-session $(loginctl | grep "$USER" | awk '{print $1}') -p Desktop | awk -F"=" 'FNR == 1  {print $NF}')
fi
echo "Window Manager is $WINDOW_MANAGER"                                       # currently supported is GNOME Shell,Mutter, Openbox,KWin


DISPLAY_MANAGER=$(basename  $(cat /etc/X11/default-display-manager) )          # currently supported is lightdm,gdm3,sddm
if [ -z "$DISPLAY_MANAGER" ]
then
      DISPLAY_MANAGER=$(loginctl show-session $(loginctl | grep "$USER" | awk '{print $1}') -p Service | awk -F"=" 'FNR == 1  {print $NF}')
fi
echo "Display Manager is $DISPLAY_MANAGER"

#echo "Desktop Environment is  $XDG_CURRENT_DESKTOP"      #does not work from ssh  #LXDE,Ubuntu GNOME,GNOME

if grep -q Raspbian /etc/os-release; then
  echo "Hardware is Raspberry Pi"
fi

if [ -f "/etc/NetworkManager/NetworkManager.conf" ]             #restore the permanent MAC address allocation
then
  if grep -q wifi.scan-rand-mac-address /etc/NetworkManager/NetworkManager.conf; then
    sudo sed 's/.*wifi.scan-rand-mac-address.*/wifi.scan-rand-mac-address=no/' -i /etc/NetworkManager/NetworkManager.conf
  else
    printf '[device]\nwifi.scan-rand-mac-address=no\n' | sudo tee -a /etc/NetworkManager/NetworkManager.conf >/dev/null
  fi
fi

"$HOME/player2/build-scripts/install-packages-common.sh"

notify "adding user to input group to read mouseClicks for kiosk"
sudo usermod -a -G input $USER

# make sure there is swap
notify "Make sure there is swap"
sudo swapon --show

if grep -qi 'x11' <<< $DISPLAY_SERVER; then
  if (grep -qvi 'openbox' <<< $WINDOW_MANAGER); then 
      notify "WM is not openbox, copying pisignage.desktop"
      mkdir -p ~/.config/autostart
      cp $HOME/player2/build-scripts/components/pisignage.desktop ~/.config/autostart
  fi

  if grep -qi 'openbox' <<< $WINDOW_MANAGER; then
      "$HOME/player2/build-scripts/openbox-keyboard-setup.sh"
  fi

  if grep -qi 'GNOME' <<< $WINDOW_MANAGER; then
      "$HOME/player2/build-scripts/gnome-keyboard-setup.sh"
  fi

  if grep -qi 'mutter\|metacity\|KWin' <<< $WINDOW_MANAGER; then
      "$HOME/player2/build-scripts/mutter-keyboard-setup.sh"
  fi

  if grep -qi 'lightdm' <<< $DISPLAY_MANAGER; then
      "$HOME/player2/build-scripts/lightdm-setup.sh"
  fi

  if grep -qi 'gdm' <<< $DISPLAY_MANAGER || grep -qi 'GNOME' <<< $WINDOW_MANAGER ; then
      "$HOME/player2/build-scripts/gdm-setup.sh"
  fi

  if grep -qi 'sddm' <<< $DISPLAY_MANAGER; then
      "$HOME/player2/build-scripts/sddm-setup.sh"
  fi
elif  grep -qi 'wayland' <<< $DISPLAY_SERVER; then
  if grep -qi 'LXDE-pi-wayfire' <<< $WINDOW_MANAGER; then
      "$HOME/player2/build-scripts/wayfire-setup.sh"
  fi
fi

if grep -q Raspbian /etc/os-release; then
  "$HOME/player2/build-scripts/raspberrypi-setup.sh"
fi




