#!/bin/bash
notify(){
    printf "\n"
    figlet -f digital -c $1
    printf "\n"
}

makeAutoLogin(){
    # Reference: https://wiki.archlinux.org/title/SDDM
    # check if /etc/sddm.conf.d directory exists, if not create directory
    notify "Enable Auto Login to Debian system"

    [ ! -d "/etc/sddm.conf.d" ] && sudo mkdir -p /etc/sddm.conf.d

    if [ ! -f /etc/sddm.conf.d/autologin.conf ]; then
        echo "creating a new autologin file"
        sudo echo "[Autologin]
#User=$USER
#Session=plasma" | sudo tee /etc/sddm.conf.d/autologin.conf
    fi

    echo "Modifying the autologin file"
    sudo sed -i "s/^\#User=.*/User=$USER/" /etc/sddm.conf.d/autologin.conf
    sudo sed -i "s/^\#Session.*/Session=plasma/" /etc/sddm.conf.d/autologin.conf
}

disableKWallet(){
    # Reference: https://wiki.archlinux.org/title/KDE_Wallet#Unlock_KDE_Wallet_automatically_on_login
    notify "Disable KWallet popup"
    if [ ! -f ~/.config/kwalletrc ]; then
        echo "creating a new kwalletrc file"
        sudo echo "[Wallet]
#Enabled=false" | sudo tee ~/.config/kwalletrc
    fi

    echo "Modifying the kwalletrc file"
    sudo sed -i "s/^\#Enabled=.*/Enabled=false/" ~/.config/kwalletrc
}

disableAutoScreenLock(){
    # Reference: https://askubuntu.com/questions/803629/how-do-i-programmatically-disable-the-kde-screen-locker
    notify "Disable Auto Screen lock"
    sudo apt install libkf5config-bin -y
    kwriteconfig5 --file kscreenlockerrc --group Daemon --key Autolock false

    # Alternatively modify ~/.config/kscreenlockerrc file with
    # [Daemon]
    # Autolock=false
}

hideWindowPanels(){
    # hide bottom panel / taskbar
    notify "Hide Window Panels like bottom taskbar"
    qdbus org.kde.plasmashell /PlasmaShell evaluateScript "p = panelById(panelIds[0]); p.height = 32 - p.height;"

    output=$(sed -n 's/.*\[PlasmaViews\]\(.*\)\[Defaults\].*/\1/p' ~/.config/plasmashellrc)
    # output might have multiple panels like [Panel 72], [Panel 100], hide all of them by looping through them
    # Loop through each line of output
    echo "$output" | while read line; do
        # escape characters should be added before [,]. Hence [Panel 72] -> \[Panel 72\]
        line_esc=$(sed 's/[][]/\\&/g'  <<< "$line")
        echo "line_esc: $line_esc";

        # check = [PlasmaViews][Panel 72][Defaults] -> the line before which the addString variable should be appended
        check="\[PlasmaViews\]$line_esc\[Defaults\]"
        echo "Check string:$check";

        # addString = [PlasmaViews][Panel 72]
        #             panelVisibility=2
        # This addString has to be added before check, 0 -> autoHide, 2 -> send panel behind window
        addString="\[PlasmaViews\]${line_esc}\\
panelVisibility=2\\
"
        echo "addstring: $addString";

        #insert addString before check
        sed -i "/^$check$/i $addString" ~/.config/plasmashellrc

        #syntax: sed -i "/^a=2/i d=4" file-name -> insert d=4 before all occurences of a=2

    done

}

hideWindowTitleBar(){

    notify "Hide WindowTitle Bar"

    # empty the contents of the file ~/.config/kwinrulesrc
    cat /dev/null > ~/.config/kwinrulesrc

    #write to ~/.config/kwinrulesrc
    echo "[1]
Description=no-border
decocolor=Breeze
decocolorrule=1
noborder=true
noborderrule=2
wmclass=.*
wmclassmatch=3

[General]
count=1" | sudo tee ~/.config/kwinrulesrc

}

setPiSignageWallpaper(){

    notify "Setting piSignage-logo wallpaper"

    # Reference: https://superuser.com/questions/488232/how-to-set-kde-desktop-wallpaper-from-command-line

    backgroundImage="$HOME/player2/public/img/pisignage-logo.png"

    dbus-send --session --dest=org.kde.plasmashell --type=method_call /PlasmaShell org.kde.PlasmaShell.evaluateScript 'string:
    var Desktops = desktops();
    for (i=0;i<Desktops.length;i++) {
            d = Desktops[i];
            d.wallpaperPlugin = "org.kde.image";
            d.currentConfigGroup = Array("Wallpaper",
                "org.kde.image",
                "General");
            d.writeConfig("Image", "file://'''$backgroundImage'''");
    }'
}

disableScreensaver(){

    notify "Disable Screensaver by modifying start.sh"
    ADDLINES=$(cat <<- END

xset s off          # Don't activate screensaver
xset -dpms          # Disable DPMS (Energy Star) features
xset s noblank      # Don't blank the video device

END
)
    echo "adding xbindkeys and screensaver-disable commands to start.sh"
    printf "xbindkeys" | sudo tee -a  $1
    printf "${ADDLINES}" | sudo tee -a $1
    # xset q command can be used to verify the settings
}

misc(){
    notify "Specific to SDDM"
    # while using pre-installed chromium, create a symlink from chromium-browser to chromium
    cd /usr/bin
    sudo ln -s chromium chromium-browser
}


notify "Setting up piSignage for SDDM-KDE"

makeAutoLogin
disableKWallet
disableAutoScreenLock
hideWindowPanels
hideWindowTitleBar
setPiSignageWallpaper
disableScreensaver $HOME/start.sh
misc