#!/bin/bash

FUNCTION=$1

CONFIG="/boot/config.txt"
HEIGHT=""
WIDTH=""
DNS_ENTRY_FILE="/etc/resolv.conf.head"
HOME_DIR=$(sudo find / -path "*/player2/package.json" 2> /dev/null | sed -e "s|/player2/package.json$||")

Change_Hostname() {
  if grep -qi 'raspbian.*bookworm' /etc/os-release; then
    sudo raspi-config nonint do_hostname "$1"
  else
    DHCLIENT="/etc/dhcp/dhclient.conf"
    HOSTS="/etc/hosts"
    newname=$1

    echo " Change Host Name To :  $newname"

    sudo sed -i "s/.*127.0.1.1.*/127.0.1.1	$newname/" $HOSTS
    sudo sh -c "echo $newname > /etc/hostname"

    sudo hostname $newname

    #sudo sed -i "s/.*send host-name \".*/send host-name \"$newname\";/" $DHCLIENT
    #sudo sed -i "s/^send host-name =.*/#send host-name = gethostname();/" $DHCLIENT
  fi
}

downgrade() {
  ####################################################################
  # this script will take care of restoring the old image in case of issue with new one
  #move the previous image
  #reboot
  #####################################################################

  pm2 stop piSignage
  killall -s 9 chromium
  killall -s 9 feh
  killall -s 9 chromium-browser
  pkill -o chromium
  pkill -o chrom

  cd $HOME

  echo "rolling back to previous release"

  if [ -d "$HOME/player2.prev"  ]
  then
      sudo rm -rf  $HOME/player2.problem
      mv  $HOME/player2  $HOME/player2.problem
      cp -a  $HOME/player2.prev/  $HOME/player2
      sync
      sudo systemctl reboot
  fi


}

Change_Dns() {
  echo "*** Change DNS To : $1 $2 for $3"
  if nmcli general status; then
    INTERFACE_NAME=$3
    CONNECTION_NAME=$(nmcli  -f DEVICE,CONNECTION device | awk -v con=$INTERFACE_NAME '$1==con{print $2}')
    sudo nmcli con mod $CONNECTION_NAME ipv4.dns "$1 $2"
  else
    # sudo mv  /etc/resolv.conf  /etc/resolv.conf.bak
    # echo -e "nameserver $1 \nnameserver $2" >> ${HOME_DIR}/resolv.conf
    # sudo mv ${HOME_DIR}/resolv.conf /etc/resolv.conf
    if [ -z $1 ]; then
      return 0
    fi

    NAME_SERVER_STR=''

    # check first entry
    if [ ! -z "$1" ]; then
      NAME_SERVER_STR="nameserver $1"
    fi

    # append second entry if present
    if [ ! -z "$2" ]; then
      NAME_SERVER_STR="${NAME_SERVER_STR}\nnameserver $2"
    fi

    echo -e "${NAME_SERVER_STR}" | sudo tee "${DNS_ENTRY_FILE}"
    return 0
  fi
}

Factory_Reset() {
  #change dispaly settings
  echo "*** Reset Dispaly Settings ***"
  sudo sed 's/.*disable_overscan.*/disable_overscan=1/' -i /boot/config.txt
  sudo sed 's/.*overscan_scale.*/overscan_scale=0/' -i /boot/config.txt

  sudo sed 's/.*hdmi_force_hotplug.*/hdmi_force_hotplug=1/' -i /boot/config.txt
  #sudo sed -i -e 's/.*overscan_left.*/\#overscan_left=40/' -e 's/.*overscan_right.*/\#overscan_right=40/' -e 's/.*overscan_top.*/\#overscan_top=20/' -e 's/.*overscan_bottom.*/\#overscan_bottom=20/' -e 's/.*framebuffer_width.*/framebuffer_width=1280/' -e 's/.*framebuffer_height.*/framebuffer_height=720/'  /boot/config.txt

  sudo sed -i 's/.*XKBLAYOUT=.*/XKBLAYOUT="us"/' /etc/default/keyboard

  # remove settings and config
  echo "*** Reset config/setting ***"
  sudo rm -rf ${HOME_DIR}/player2/config/_device-settings.json
  sudo rm -rf ${HOME_DIR}/player2/config/_media-settings.json

  # change ht passwd file
  echo 'pi:$apr1$G4yYdggH$aqJwlrg6yXYERl4DCZ2DT1' >${HOME_DIR}/player2/htpasswd

  Change_Hostname "raspberrypi"
  echo "*** Changed name to  $HOSTNAME ***"
}

One_Time() {
  # script to add execute permission
  chmod -R +x shell-scripts/upgrade_scripts/*
  node pi-upgrade.js
}

Change_SSH_Password() {
  # Checked to work with player2
  echo "Changing SSH password"

  NEW_PASSWORD=$1
  if [ -z $NEW_PASSWORD ]; then
    echo "Password empty"
    exit 1
  fi

  USER_NAME=$(id -u -n)
  if [ -z $USER_NAME ]; then
    echo "USER NAME empty"
    exit 1
  fi

  sudo passwd ${USER_NAME} <<-EOF
		${NEW_PASSWORD}
		${NEW_PASSWORD}
	EOF
}
new_wifi_entry() {
  if nmcli general status; then
    # AP_CON_NAME="HIDDEN_WIFI"
    WIFI_INTERFACE=$(sudo iw dev | awk '$1=="Interface"{print $2}')
    echo "Wifi Interface: ${WIFI_INTERFACE}"
    INTERFACE_NAME=$4
    echo "Interface name: ${INTERFACE_NAME}"
    # check if the Connection name exists in the list of wlan0 connections
    CONNECTION_NAME=$(nmcli -t -f NAME,DEVICE connection | awk -v con="${INTERFACE_NAME}" -F ':' '$1==con{print $1}' | head -n 1)
    echo "ConnectionName Exists: ${CONNECTION_NAME}"
    sudo raspi-config nonint do_wifi_country "${2}"
#    AP_CONNECTION_EXISTS=$(sudo nmcli connection show $CONNECTION_NAME)
#    if [[ -z "${AP_CONNECTION_EXISTS}" ]] || [[ ${AP_CONNECTION_EXISTS} != *"Error"* ]]; then
    # if [[ ! -z ${CONNECTION_NAME} ]] && [[ $(sudo nmcli -f GENERAL.STATE connection show "$CONNECTION_NAME") ]]; then
    if [[ ! -z ${CONNECTION_NAME} ]]; then
      if [ ! -z ${5} ]; then
        sudo nmcli con modify "${CONNECTION_NAME}" wifi-sec.key-mgmt wpa-psk wifi-sec.psk "${5}"
      fi
      sudo nmcli con up "${CONNECTION_NAME}"
      echo "WiFi Connection already exists, modifying password and Activating it"
      return 0
    fi
    CONNECTION_NAME=$4
    if [ $3 = true ]; then
      sudo nmcli con add type wifi con-name "$CONNECTION_NAME" ifname ${WIFI_INTERFACE} ssid "${CONNECTION_NAME}" hidden yes
    else
      sudo nmcli con add type wifi con-name "$CONNECTION_NAME" ifname ${WIFI_INTERFACE} ssid "${CONNECTION_NAME}"
    fi
    if [ ! -z ${5} ]; then
      sudo nmcli con modify "$CONNECTION_NAME" wifi-sec.key-mgmt wpa-psk wifi-sec.psk "${5}"
    fi
    sudo nmcli con up "$CONNECTION_NAME"

    echo "Added new WiFi Connection and Activated it"

    return 0

    # Additional commands
    # sudo nmcli con down <connection-name> # disconnect from WiFi
    # sudo nmcli con delete <connection-name>  # Forget WiFi connection
    # Example: new_wifi_entry "" "" false "Pixel 6a" "pisignageTest"
  else
   WIFI_SUPPLICANT=/etc/wpa_supplicant/wpa_supplicant.conf

   sudo sed -i '/update_config=1/q0' $WIFI_SUPPLICANT
   sudo echo "country=${2} " >>$WIFI_SUPPLICANT

   sudo echo "network={" >>$WIFI_SUPPLICANT
   sudo echo "  ssid=\"${4}\" " >>$WIFI_SUPPLICANT
   if [ -z ${5} ]; then
     sudo echo "  key_mgmt=NONE " >>$WIFI_SUPPLICANT
   else
     sudo echo "  psk=\"${5}\" " >>$WIFI_SUPPLICANT
   fi

   if [ $3 = true ]; then
     sudo echo "  scan_ssid=1 " >>$WIFI_SUPPLICANT
   fi

   sudo echo "}" >>$WIFI_SUPPLICANT

   sudo wpa_cli -i ${1} reconfigure

   return 0
  fi
}

do_ip_settings() {
  if nmcli general status; then
    INTERFACE_NAME=$1
    # CONNECTION_NAME=$(nmcli  -f DEVICE,CONNECTION device | awk -v con=$INTERFACE_NAME '$1==con{print $2}')
    CONNECTION_NAME=$(nmcli -t -f NAME,DEVICE connection show | awk -v interface="${INTERFACE_NAME}" -F ':' '$2==interface{print $1}')
    if [[ ! -z "${2}" ]] && [[ ! -z "${3}" ]] && [[ ! -z "${4}" ]]; then    #STATIC
      sudo nmcli con mod "$CONNECTION_NAME" ipv4.addresses $2 
      sudo nmcli con mod "$CONNECTION_NAME" ipv4.method manual
      sudo nmcli con mod "$CONNECTION_NAME" ipv4.gateway $3
      sudo nmcli con mod "$CONNECTION_NAME" ipv4.dns $4
    else   #DHCP
      sudo nmcli con mod "${CONNECTION_NAME}" ipv4.method auto
      sudo nmcli con mod "${CONNECTION_NAME}" ipv4.gateway ""
      sudo nmcli con mod "${CONNECTION_NAME}" ipv4.address ""
      sudo nmcli con mod "${CONNECTION_NAME}" ipv4.dns ""
    fi
    nmcli con down "$CONNECTION_NAME"
    nmcli con up "$CONNECTION_NAME"
  else
    DHCPCD_FILE=/etc/dhcpcd.conf
    echo "Deleting static IP lines in $DHCPCD_FILE if present"
  #  if grep -q "^interface ${1}" $DHCPCD_FILE; then
  #    sed -i "/^interface ${1}/,+4d" ${DHCPCD_FILE}
  #  fi
    sudo sed -i "/^interface ${1}/,+4d" ${DHCPCD_FILE}

    if [[ ! -z "${2}" ]] && [[ ! -z "${3}" ]] && [[ ! -z "${4}" ]]; then
      echo "interface ${1}" | sudo tee -a ${DHCPCD_FILE}
      echo "      static ip_address=${2}" | sudo tee -a ${DHCPCD_FILE}
      echo "      static routers=${3}" | sudo tee -a ${DHCPCD_FILE}
      echo "      static domain_name_servers=${4}" | sudo tee -a ${DHCPCD_FILE}
    fi
  fi
}

Enable_Access_Point() {
  if nmcli general status; then
      # Don't change if current mode is same
      WIFI_INTERFACE=$(sudo iw dev | awk '$1=="Interface"{print $2}')
      echo "Wifi Interface: ${WIFI_INTERFACE}"

      APMode=$(sudo iw dev ${WIFI_INTERFACE} info | grep type | awk '{print $2}')
      if [[ ${APMode} == "AP" ]];then
        echo "already in AP mode"
        return 0
      fi
      # check if the wifi_interface supports Access point mode
      # https://access.redhat.com/documentation/en-us/red_hat_enterprise_linux/9/html/configuring_and_managing_networking/assembly_configuring-rhel-as-a-wifi-access-point_configuring-and-managing-networking
      APSupported=$(nmcli -f WIFI-PROPERTIES.AP device show ${WIFI_INTERFACE})  #   WIFI-PROPERTIES.AP:  yes
      echo "AP Supported: ${APSupported}"

      if [[ -z "${APSupported}" ]] || [[ ${APSupported} != *"yes"* ]]; then
        echo "Access point mode is not supported, since no compatible interface found"
        return 0
      fi

      # settings Access point name
      CPUID=$(cat /sys/class/net/e[nt]*/address | sed -e "s/://g")
      ACCESS_POINT_NAME=$(echo "piplayer_${CPUID: -4}")
      AP_CON_NAME="WIFI_AP"

      echo "ACCESS_POINT_NAME: ${ACCESS_POINT_NAME}"
      echo "AP_CON_NAME: ${AP_CON_NAME}"

      AP_CONNECTION_EXISTS=$(sudo nmcli connection show ${AP_CON_NAME})
      AP_CONNECTION_EXISTS="${AP_CONNECTION_EXISTS:-NO_AP_FOUND}"

      # TODO - Change the hardCoded /home/pi to $HOME
#      echo "creating an empty file /home/pi/accessPoint.txt"
#      touch /home/pi/accessPoint.txt

      if [[ -z "${AP_CONNECTION_EXISTS}" ]] || [[ ${AP_CONNECTION_EXISTS} != "NO_AP_FOUND" ]]; then
        echo "Deleting existing Access Point connection and creating a new one"
        sudo nmcli connection delete ${AP_CON_NAME}
      fi

      # https://variwiki.com/index.php?title=Wifi_NetworkManager#Configuring_WiFi_Access_Point
      sudo nmcli con add type wifi ifname ${WIFI_INTERFACE} mode ap con-name ${AP_CON_NAME} ssid ${ACCESS_POINT_NAME} autoconnect yes

      # https://developer-old.gnome.org/NetworkManager/stable/settings-802-11-wireless.html
      # https://developer-old.gnome.org/NetworkManager/stable/settings-802-11-wireless-security.html
      sudo nmcli con modify ${AP_CON_NAME} 802-11-wireless.band bg # Use the 2.4GHz band
      sudo nmcli con modify ${AP_CON_NAME} 802-11-wireless.channel 6 # use channel 6
      sudo nmcli con modify ${AP_CON_NAME} 802-11-wireless-security.key-mgmt wpa-psk
      sudo nmcli con modify ${AP_CON_NAME} 802-11-wireless-security.psk piplayer
      sudo nmcli con modify ${AP_CON_NAME} ipv4.method shared # If 'shared' is specified (indicating that this connection will provide network access to other computers) then the interface is assigned an address in the 10.42.x.1/24 range and a DHCP and forwarding DNS server are started, and the interface is NAT-ed to the current default network connection
      sudo nmcli con modify ${AP_CON_NAME} ipv4.addr 192.168.100.1/24  # Adding wifi static IP
      sudo nmcli con modify ${AP_CON_NAME} ipv6.method disabled
      # sudo nmcli con modify ${AP_CON_NAME} ipv6.dns 8.8.8.8  # TODO - Bookworm release
      sudo nmcli con up ${AP_CON_NAME}

      echo "Wi-fi in Access Point Mode"

      # Additional commands
      # nmcli device wifi # Check the AP configurating using this command
      # sudo nmcli connection delete uuid <UUID> # delete a connection using this command

  else
     # Don't change if current mode is same
     # WIFI_INTERFACE=$(sudo ifconfig -a | sed -e 's/[ \t].*//;/^$/d' -e 's/://' | grep wl)
     WIFI_INTERFACE=$(sudo iw dev | awk '$1=="Interface"{print $2}')
     if grep -q "^denyinterfaces ${WIFI_INTERFACE}" /etc/dhcpcd.conf; then
       echo "already in AP mode"
       return 0
     fi

     if [ ! -f "/etc/hostapd/hostapd.conf" ]; then
       echo " ============== copy setting hostapd.conf file ===================="
       sudo cp ${HOME_DIR}/player2/shell-scripts/network/hostapd.conf /etc/hostapd/
     fi

     sudo sed -i "s/wlan0/${WIFI_INTERFACE}/g" /etc/hostapd/hostapd.conf
     sudo chmod 755 /etc/hostapd/hostapd.conf
     CPUID=$(cat /sys/class/net/e[nt]*/address | sed -e "s/://g")
     ACCESS_POINT_NAME=$(echo "${CPUID: -4}")
     sudo sed -i 's/^ssid=.*/ssid=piplayer_'$ACCESS_POINT_NAME'/' /etc/hostapd/hostapd.conf
     sudo sed -i 's|.*#DAEMON_CONF=""*.|DAEMON_CONF="/etc/hostapd/hostapd.conf"|' /etc/default/hostapd

     #if [ ! -f  "/etc/dnsmasq.conf" ];then
     sudo cp ${HOME_DIR}/player2/shell-scripts/network/dnsmasq.conf /etc/dnsmasq.conf
     sudo sed -i "s/wlan0/${WIFI_INTERFACE}/g" /etc/dnsmasq.conf
     sudo chmod 755 /etc/dnsmasq.conf
     #fi

     if grep -q "denyinterfaces ${WIFI_INTERFACE}" /etc/dhcpcd.conf; then
       sudo sed -i "s/.*denyinterfaces ${WIFI_INTERFACE}*./denyinterfaces ${WIFI_INTERFACE}/" /etc/dhcpcd.conf
     else
       echo "denyinterfaces ${WIFI_INTERFACE}" | sudo tee -a /etc/dhcpcd.conf
     fi

     echo "Adding wifi static IP info /etc/dhcpcd.conf"
     sed -i "/^interface ${WIFI_INTERFACE}/d" /etc/dhcpcd.conf
     sed -i '/static ip_address=192.168.100.1/d' /etc/dhcpcd.conf
     sed -i '/nohook wpa_supplicant/d' /etc/dhcpcd.conf
     sudo sed -i "/denyinterfaces ${WIFI_INTERFACE}/iinterface ${WIFI_INTERFACE}\n\tstatic ip_address=192.168.100.1/24\n\tnohook wpa_supplicant\n" /etc/dhcpcd.conf

     #delete the network info if present
     sudo sed -i '/network={/,+4d' /etc/wpa_supplicant/wpa_supplicant.conf

     # Start And enable service
     sudo systemctl enable dnsmasq.service
     sudo systemctl start dnsmasq.service

     sudo systemctl unmask hostapd.service
     sudo systemctl enable hostapd.service
     sudo systemctl daemon-reload

     sudo systemctl start hostapd.service
     echo "Wi-fi in Access Point Mode"
   fi
}

Disable_Access_Point() {
  if nmcli general status; then
      # WIFI_INTERFACE=$(sudo ifconfig -a | sed -e 's/[ \t].*//;/^$/d' -e 's/://' | grep wl)
      WIFI_INTERFACE=$(sudo iw dev | awk '$1=="Interface"{print $2}')
      echo "Wifi Interface: ${WIFI_INTERFACE}"

      # if disableAccessPoint from Group Settings or Ctrl+N AP Config, delete the file
#      if [[ $1 = true ]]; then
#          echo "Removing /home/pi/accessPoint.txt"
#          sudo rm -rf /home/pi/accessPoint.txt
#      fi
      # sudo nmcli con modify ${AP_CON_NAME} connection.autoconnect no
      # sudo nmcli con modify ${AP_CON_NAME} connection.autoconnect-priority 0

      # Don't change if current mode is same - if "new_wifi_entry" is called before disableAP, then it will still be reported as Managed by WIFI_AP will be inactive mode
      # APMode=$(sudo iw dev wlan0 info | grep type | awk '{print $2}')
      # if [[ ${APMode} != "AP" ]];then
      #   echo "already in  Wifi mode"
      #   return 0
      # fi

      AP_CON_NAME="WIFI_AP"
      sudo nmcli con delete ${AP_CON_NAME}

      echo "Access Point mode disabled"
  else
       # WIFI_INTERFACE=$(sudo ifconfig -a | sed -e 's/[ \t].*//;/^$/d' -e 's/://' | grep wl)
       WIFI_INTERFACE=$(sudo iw dev | awk '$1=="Interface"{print $2}')
       # Don't change if current mode is same
       if grep -q "^\#denyinterfaces ${WIFI_INTERFACE}" /etc/dhcpcd.conf; then
         echo "already in  Wifi mode"
         return 0
       fi
       # Stop And Disable service
       sudo ifdown ${WIFI_INTERFACE}
       sudo systemctl stop hostapd.service
       sudo systemctl stop dnsmasq.service

       sudo systemctl disable hostapd.service
       sudo systemctl disable dnsmasq.service
       sudo systemctl daemon-reload

       # allow wifi interface
       sudo sed -i "s/.*denyinterfaces ${WIFI_INTERFACE}*./#denyinterfaces ${WIFI_INTERFACE}/" /etc/dhcpcd.conf

       echo "Deleting ${WIFI_INTERFACE} static IP info /etc/dhcpcd.conf"
       sudo sed -i "/^interface ${WIFI_INTERFACE}/d" /etc/dhcpcd.conf
       sudo sed -i '/static ip_address=192.168.100.1/d' /etc/dhcpcd.conf
       sed -i '/nohook wpa_supplicant/d' /etc/dhcpcd.conf
       echo "Access Point mode disabled"
   fi
}

Enable_4k() {
  #By default, when connected to a 4K monitor, the Raspberry Pi 4B, 400 and CM4 will select a 30Hz refresh rate.
  #Use this option to allow selection of 60Hz refresh rates.
  #Setting hdmi_enable_4kp60 will increase power consumption and the temperature of your Raspberry Pi.only 1 HDMI port can be used
  if grep -qi 'raspbian.*bullseye' /etc/os-release; then
    if grep -q hdmi_enable_4kp60 /boot/config.txt; then
      sudo sed 's/.*hdmi_enable_4kp60.*/hdmi_enable_4kp60=1/' -i /boot/config.txt
    else
      echo 'hdmi_enable_4kp60=1' | sudo tee -a /boot/config.txt >/dev/null
    fi
    #https://forums.raspberrypi.com/viewtopic.php?p=169726#p169726
    if grep -q force_turbo /boot/config.txt; then
      sudo sed 's/.*force_turbo.*/force_turbo=1/' -i /boot/config.txt
    else
      echo 'force_turbo=1' | sudo tee -a /boot/config.txt >/dev/null
    fi
  else
    sudo raspi-config nonint do_pi4video V1
    sudo raspi-config nonint do_overclock Turbo
  fi
}

Disable_4k() {
  if grep -qi 'raspbian.*bullseye' /etc/os-release; then
    if grep -q hdmi_enable_4kp60 /boot/config.txt; then
      sudo sed 's/.*hdmi_enable_4kp60.*/#hdmi_enable_4kp60=1/' -i /boot/config.txt
    else
      echo '#hdmi_enable_4kp60=1' | sudo tee -a /boot/config.txt >/dev/null
    fi
    if grep -q force_turbo /boot/config.txt; then
      sudo sed 's/.*force_turbo.*/#force_turbo=1/' -i /boot/config.txt
    else
      echo '#force_turbo=1' | sudo tee -a /boot/config.txt >/dev/null
    fi
  else
    sudo raspi-config nonint do_pi4video V3
    sudo raspi-config nonint do_overclock None
  fi
}

show_help() {
  cat <<-EOF
    USAGE:  /bin/bash system.sh [OPTIONS=VALUE]

            Currently Multiple options are not allowed. You can use only single option
            Ex: /bin/bash system --resolution=720p portrait

    OPTIONS:

        --change-hostname
                        Used to change hostname of your raspberry pi
                        ex: /bin/bash system --change-hostname=mycomputer
        --factory-reset
                        Used to perform factory reset of raspberry pi
                        All the configuration will be set to default values, That is screen at 720p,
                        orientation at  Landscape
                        hostname changed to raspberrypi
                        ethernet set to DHCP mode
                        ex: /bin/bash system --factory-reset
        --enable-4k
                        Enable 4K
        --disable-4k
       	                Disable 4K
        --enable-access-point
                        Make raspberry pi as access point device
        --disable-access-point
       	                Disable access point feature in Raspberry Pi
        --change-dns
                        This option will change the dns entry.
                        ex: /bin/bash system --change-dns 8.8.4.4 8.8.8.8 <interface>
        --setup-wifi
                        Set up wifi for the device
                        ex: /bin/bash system --setup-wifi <interface> <country> <hidden> <ssid> <psk> 
        --ipconfig
                        Configure Static IP or DHCP
                        ex: /bin/bash system --ipconfig <interface> for DHCP
                            /bin/bash system --ipconfig <interface> <ipadrres> <gateway> <dns> for Static IP

EOF

}

case "$FUNCTION" in
--change-hostname=*)
  NAME=$(echo $1 | sed 's/[-a-zA-Z0-9]*=//; s/ /-/g; s/^-//; s/[^-a-zA-Z0-9]//g')
  Change_Hostname $NAME
  ;;
--factory-reset)
  Factory_Reset
  ;;
--change-dns)
  Change_Dns $2 $3 $4
  ;;
--one-time)
  One_Time
  ;;
--setup-wifi)
  new_wifi_entry $2 $3 $4 "$5" "$6" 
  ;;
--ipconfig)
  do_ip_settings $2 $3 $4 $5
  ;;
--enable-4k)
  Enable_4k
  ;;
--disable-4k)
  Disable_4k
  ;;
--enable-access-point)
  Enable_Access_Point
  ;;
--disable-access-point)
  Disable_Access_Point
  ;;
--change-ssh-password)
  Change_SSH_Password $2
  ;;
--clean-chrome-session)
  Clean_Chrome_Session
  ;;
--change-config-txt)
  Change_Config_Txt $2 $3
  ;;
*)
  echo "**** Check Your Command Line Options *** "
  echo ""
  echo ""
  show_help
  exit 1
  ;;
esac
